@@ -1,6 +1,6 @@
 /* Processing functions used by all sexpr processing functions.
 
-   Copyright (C) 2012 Ian Dunn.
+   Copyright (C) 2012, 2013 Ian Dunn.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
@@ -21,6 +21,204 @@
 #include "vec.h"
 #include "var.h"
 #include "list.h"
+#include <stdarg.h>
+
+/* Gets a sentence part from a sexpr.
+ *  input:
+ *   in_str - the input string.
+ *   init_pos - the initial position within in_str.
+ *   out_str - the string that receives the new sentence part, if any.
+ *  output:
+ *   the position in in_str of the end of the part, or -1 on memory error.
+ */
+int
+sexpr_get_part (unsigned char * in_str, unsigned int init_pos, unsigned char ** out_str)
+{
+  int fin_pos = init_pos;
+  int tmp_pos;
+
+  *out_str = NULL;
+
+  switch (in_str[init_pos])
+    {
+    case '(':
+      tmp_pos = parse_parens (in_str, init_pos, out_str);
+      if (tmp_pos == -2)
+	return -1;
+
+      fin_pos = tmp_pos + 1;
+      break;
+
+    case ' ':
+      fin_pos++;
+      break;
+
+    default:
+      tmp_pos = init_pos;
+      while (in_str[tmp_pos] != ' '
+	     && in_str[tmp_pos] != ')'
+	     && in_str[tmp_pos] != '\0')
+	tmp_pos++;
+
+      (*out_str) = (unsigned char *) calloc (tmp_pos - init_pos + 1, sizeof (char));
+      CHECK_ALLOC ((*out_str), -1);
+      strncpy ((*out_str), in_str + init_pos, tmp_pos - init_pos);
+      (*out_str)[tmp_pos - init_pos] = '\0';
+
+      fin_pos = tmp_pos;
+      break;
+    }
+
+  return fin_pos;
+}
+
+unsigned char *
+sexpr_car (unsigned char * in_str)
+{
+  int init_pos = (in_str[0] == '(') ? 1 : 0;
+  unsigned char * out_str;
+  int pos;
+  pos = sexpr_get_part (in_str, 1, &out_str);
+  if (pos == -1)
+    return NULL;
+
+  return out_str;
+}
+
+unsigned char *
+sexpr_cdr (unsigned char * in_str)
+{
+  unsigned char * out_str, * car;
+  int pos, len;
+  pos = sexpr_get_part (in_str, 1, &car);
+  if (pos == -1)
+    return NULL;
+
+  len = strlen (in_str);
+  out_str = (unsigned char *) calloc (len - pos + 1, sizeof (char));
+  CHECK_ALLOC (out_str, NULL);
+
+  sprintf (out_str, "(%s", in_str + pos + 1);
+
+  return out_str;
+}
+
+int
+sexpr_car_cdr (unsigned char * in_str,
+	       unsigned char ** car,
+	       vec_t * cdr)
+{
+  int init_pos = (in_str[0] == '(') ? 1 : 0;
+  int pos, ret_chk;
+  pos = sexpr_get_part (in_str, init_pos, car);
+  if (pos == -1)
+    return -1;
+
+  pos++;
+
+  while (in_str[pos] != '\0')
+    {
+      int tmp_pos;
+      unsigned char * tmp_str = NULL;
+
+      tmp_pos = sexpr_get_part (in_str, pos, &tmp_str);
+      if (tmp_pos == -1)
+	return -1;
+
+      if (tmp_str)
+	{
+	  ret_chk = vec_str_add_obj (cdr, tmp_str);
+	  if (ret_chk == -1)
+	    return -1;
+	}
+      pos = tmp_pos + 1;
+    }
+
+  return cdr->num_stuff;
+}
+
+int
+sexpr_str_car_cdr (unsigned char * in_str,
+		   unsigned char ** car,
+		   unsigned char ** cdr)
+{
+  int tmp_pos, ret_chk;
+
+  *car = *cdr = NULL;
+
+  tmp_pos = sexpr_get_part (in_str, 1, car);
+  if (tmp_pos == -1)
+    return -1;
+
+  tmp_pos += 1;
+
+  tmp_pos = parse_parens (in_str, tmp_pos, cdr);
+  if (tmp_pos == -2)
+    return -1;
+
+  if (tmp_pos < 0)
+    return -2;
+
+  return 0;
+}
+
+/* Places sentences based on their lengths.
+ *  input:
+ *    in0 - the first sentence to place.
+ *    in1 - the second sentence to place.
+ *    sh_sen - a pointer to the shorter sentence.
+ *    ln_sen - a pointer to the longer sentence.
+ *  output:
+ *    none
+ */
+void
+sen_put_len (unsigned char * in0,
+	     unsigned char * in1,
+	     unsigned char ** sh_sen,
+	     unsigned char ** ln_sen)
+{
+  int len0, len1;
+  len0 = strlen (in0);
+  len1 = strlen (in1);
+
+  if (len0 > len1)
+    {
+      *sh_sen = in1;
+      *ln_sen = in0;
+    }
+  else
+    {
+      *sh_sen = in0;
+      *ln_sen = in1;
+    }
+}
+
+unsigned char *
+construct_other (unsigned char * main_str,
+		 int init_pos,
+		 int fin_pos,
+		 int alloc_size,
+		 char * template,
+		 ...)
+{
+  unsigned char * oth_sen;
+  int oth_pos = init_pos;
+  va_list args;
+
+  va_start (args, template);
+
+  oth_sen = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+  CHECK_ALLOC (oth_sen, NULL);
+  strncpy (oth_sen, main_str, oth_pos);
+
+  oth_pos += vsprintf (oth_sen + oth_pos, template, args);
+
+  strcpy (oth_sen + oth_pos, main_str + fin_pos);
+
+  va_end (args);
+
+  return oth_sen;
+}
 
 /* Checks for a negation on a sexpr string.
  *  input:
@@ -93,81 +291,38 @@ int
 sexpr_get_generalities (unsigned char * in_str, unsigned char * conn, vec_t * vec)
 {
   int ret_chk;
-  int in_len;
+  unsigned char * tmp_conn;
 
-  if (in_str[0] != '(' || in_str[1] == '(')
+  ret_chk = sexpr_car_cdr (in_str, &tmp_conn, vec);
+  if (ret_chk == -1)
+    return -1;
+
+  if (ret_chk == 0)
     {
-      ret_chk = vec_str_add_obj (vec, in_str);
-      if (ret_chk < 0)
-	return -1;
-
+      vec_str_add_obj (vec, tmp_conn);
       return 1;
     }
 
-  int pos = 1;
+  if (tmp_conn[0] == '(')
+    {
+      return 1;
+    }
 
   if (conn[0] == '\0')
     {
-      strncpy (conn, in_str + pos, S_CL);
+      strncpy (conn, tmp_conn, S_CL);
       conn[S_CL] = '\0';
     }
   else
     {
-      if (strncmp (conn, in_str + pos, S_CL))
+      if (strncmp (conn, tmp_conn, S_CL))
 	{
-	  ret_chk = vec_str_add_obj (vec, in_str);
-	  if (ret_chk < 0)
-	    return -1;
-
+	  free (tmp_conn);
 	  return 1;
 	}
     }
 
-  pos += S_CL + 1;
-
-  in_len = strlen (in_str);
-  while (in_str[pos] != ')')
-    {
-      int tmp_pos;
-      unsigned char * tmp_str;
-
-      switch (in_str[pos])
-	{
-	case '(':
-	  tmp_pos = parse_parens (in_str, pos, &tmp_str);
-	  if (tmp_pos == -2)
-	    return -1;
-
-	  ret_chk = vec_str_add_obj (vec, tmp_str);
-	  if (ret_chk < 0)
-	    return -1;
-
-	  pos = tmp_pos + 1;
-	  break;
-
-	case ' ':
-	  pos++;
-	  break;
-
-	default:
-	  tmp_pos = pos;
-	  while (in_str[tmp_pos] != ' ' && in_str[tmp_pos] != ')')
-	    tmp_pos++;
-
-	  tmp_str = (unsigned char *) calloc (tmp_pos - pos + 1, sizeof (char));
-	  CHECK_ALLOC (tmp_str, -1);
-	  strncpy (tmp_str, in_str + pos, tmp_pos - pos);
-	  tmp_str[tmp_pos - pos] = '\0';
-
-	  ret_chk = vec_str_add_obj (vec, tmp_str);
-	  if (ret_chk < 0)
-	    return -1;
-
-	  pos = tmp_pos;
-	  break;
-	}
-    }
-
+  free (tmp_conn);
   return vec->num_stuff;
 }
 
@@ -187,6 +342,8 @@ sexpr_find_top_connective (unsigned char * in_str, unsigned char * conn,
   int gg;
   vec_t * vec;
 
+  *lsen = *rsen = NULL;
+
   vec = init_vec (sizeof (char *));
   if (!vec)
     return -1;
@@ -319,73 +476,21 @@ int
 sexpr_get_pred_args (unsigned char * in_str, unsigned char ** pred, vec_t * vec)
 {
   int ret_chk;
+  unsigned char * tmp_pred;
 
-  if (in_str[0] != '(' || in_str[1] == '(')
+  ret_chk = sexpr_car_cdr (in_str, &tmp_pred, vec);
+  if (ret_chk == -1)
+    return -1;
+
+  if (tmp_pred[0] == '(' || ret_chk == 0)
     return 0;
 
-  int pos, oth_pos;
-
-  pos = oth_pos = 1;
-  while (in_str[oth_pos] != ' ')
-    oth_pos++;
-
   if (pred)
     {
-      *pred = (unsigned char *) calloc (oth_pos - pos + 1, sizeof (char));
-      CHECK_ALLOC (*pred, -1);
-
-      strncpy (*pred, in_str + pos, oth_pos - pos);
-      (*pred)[oth_pos - pos] = '\0';
-    }
-
-  unsigned char * elm_str;
-  elm_str = elim_par (in_str);
-  if (!elm_str)
-    return -1;
-
-  pos = oth_pos - 1;
-
-  while (elm_str[pos] != '\0')
-    {
-      int tmp_pos;
-      unsigned char * tmp_str;
-
-      switch (elm_str[pos])
-	{
-	case '(':
-	  tmp_pos = parse_parens (elm_str, pos, &tmp_str);
-	  if (tmp_pos == -2)
-	    return -1;
-
-	  ret_chk = vec_str_add_obj (vec, tmp_str);
-	  if (ret_chk < 0)
-	    return -1;
-	  free (tmp_str);
-
-	  pos = tmp_pos + 1;
-	  break;
-
-	case ' ':
-	  pos++;
-	  break;
-
-	default:
-	  tmp_pos = pos;
-	  while (elm_str[tmp_pos] != ' ' && elm_str[tmp_pos] != '\0')
-	    tmp_pos++;
-
-	  tmp_str = (unsigned char *) calloc (tmp_pos - pos + 1, sizeof (char));
-	  CHECK_ALLOC (tmp_str, -1);
-	  strncpy (tmp_str, elm_str + pos, tmp_pos - pos);
-	  tmp_str[tmp_pos - pos] = '\0';
-
-	  ret_chk = vec_str_add_obj (vec, tmp_str);
-	  if (ret_chk < 0)
-	    return -1;
-	  free (tmp_str);
-	  pos = tmp_pos;
-	  break;
-	}
+      *pred = strdup (tmp_pred);
+      if (!(*pred))
+	return -1;
+      free (tmp_pred);
     }
 
   return vec->num_stuff;
@@ -403,35 +508,33 @@ unsigned char *
 sexpr_elim_quant (unsigned char * in_str, unsigned char * quant,
 		  unsigned char ** var)
 {
-  int tmp_pos;
-  unsigned char * tmp_str;
 
-  tmp_pos = parse_parens (in_str, 0, &tmp_str);
-  if (tmp_pos == -2)
+  int tmp_pos, ret_chk;
+  unsigned char * tmp_str, * car;
+
+  tmp_pos = sexpr_str_car_cdr (in_str, &car, &tmp_str);
+  if (tmp_pos == -1)
     return NULL;
-  free (tmp_str);
 
-  if (in_str[tmp_pos + 1] != '\0')
+  if (tmp_pos == -2)
+    {
+      if (car) free (car);
+      if (tmp_str) free (tmp_str);
+      return "\0";
+    }
+
+  *var = NULL;
+
+  int alloc_size = strlen (car) - 3 - S_CL;
+  *var = (unsigned char *) calloc (alloc_size + 1, sizeof (char));
+  CHECK_ALLOC (var, NULL);
+
+  ret_chk = sscanf (car, "(%s %[^)])", quant, *var);
+  if (ret_chk != 2
+      || (strcmp (quant, S_UNV) && strcmp (quant, S_EXL)))
     return "\0";
 
-  tmp_pos = parse_parens (in_str, 1, &tmp_str);
-  if (tmp_pos == -2)
-    return NULL;
-  free (tmp_str);
-
-  strncpy (quant, in_str + 2, S_CL);
-  quant[S_CL] = '\0';
-
-  *var = (unsigned char *) calloc (tmp_pos - (2 + S_CL), sizeof (char));
-  CHECK_ALLOC (*var, NULL);
-  strncpy (*var, in_str + 3 + S_CL, tmp_pos - (3 + S_CL));
-
-  if (in_str[tmp_pos + 2] != '(')
-    return "\0";
-
-  tmp_pos = parse_parens (in_str, tmp_pos + 2, &tmp_str);
-  if (tmp_pos == -2)
-    return NULL;
+  free (car);
 
   return tmp_str;
 }
@@ -1010,7 +1113,8 @@ sexpr_get_ids (unsigned char * sen, int ** ids, vec_t * sen_ids)
 	  int new_id;
 
 	  start = end = i;
-	  while (isalnum (sen[end]))
+	  end++;
+	  while (ISLEGIT (sen[end]))
 	    end++;
 
 	  if (sen_ids)
